//---------------------------------------------------------------------------
#ifndef __VECTOR_H__
#define __VECTOR_H__
//---------------------------------------------------------------------------
#include <math>
//---------------------------------------------------------------------------
class Vector {
  public:
    double x, y, z;
    Vector() {
      }
    Vector( double v ):
      x( v ), y( v ), z( v ) {
      }
    Vector( const Vector& v ):
      x( v.x ), y( v.y ), z( v.z ) {
      }
    Vector( double vx, double vy, double vz ):
      x( vx ), y( vy ), z( vz ) {
      }
    Vector operator - () const {
      return Vector( -x, -y, -z );
      }
    Vector& operator = ( double a ) {
      x = y = z = a;
      return *this;
      }
    Vector& operator = ( const Vector& v ) {
      x = v.x;
      y = v.y;
      z = v.z;
      return *this;
      }
    Vector& operator += ( double v ) {
      x += v;
      y += v;
      z += v;
      return *this;
      }
    Vector& operator -= ( double v ) {
      x -= v;
      y -= v;
      z -= v;
      return *this;
      }
    Vector& operator *= ( double v ) {
      x *= v;
      y *= v;
      z *= v;
      return *this;
      }
    Vector& operator /= ( double v ) {
      x /= v;
      y /= v;
      z /= v;
      return *this;
      }
    Vector& operator *= ( const Vector& v ) {
      x *= v.x;
      y *= v.y;
      z *= v.z;
      return *this;
      }
    Vector& operator += ( const Vector& v ) {
      x += v.x;
      y += v.y;
      z += v.z;
      return *this;
      }
    Vector& operator -= ( const Vector& v ) {
      x -= v.x;
      y -= v.y;
      z -= v.z;
      return *this;
      }
    Vector& operator /= ( const Vector& v ) {
      x /= v.x;
      y /= v.y;
      z /= v.z;
      return *this;
      }
    const double operator!() const {
      return (double) sqrt( x*x + y*y + z*z );
      }

  friend const Vector operator + ( const Vector& v1, const Vector& v2 ) {
    return Vector( v1 ) += v2;
    }
  friend const Vector operator - ( const Vector& v1, const Vector& v2 ) {
    return Vector( v1 ) -= v2;
    }
  friend const Vector operator * ( const Vector& v1, const Vector& v2 ) {
    return Vector( v1 ) *= v2;
    }
  friend const Vector operator * ( const Vector& v, double a ) {
    return Vector( v ) *= a;
    }
  friend const Vector operator * ( double a, const Vector& v ) {
    return Vector( v ) *= a;
    }
  friend const Vector operator / ( const Vector& v1, const Vector& v2 ) {
    return Vector( v1 ) /= v2;
    }
  friend const Vector operator / ( const Vector& v, double a ) {
    return Vector( v ) /= a;
    }
  friend const double operator & ( const Vector& v1, const Vector& v2 ) {
    return v1.x * v2.x + v1.y * v2.y + v1.z * v2.z;
    }
  friend const Vector operator ^ ( const Vector& v1, const Vector& v2 ) {
    return Vector( v1.y * v2.z - v1.z * v2.y,
   		   v1.z * v2.x - v1.x * v2.z,
		   v1.x * v2.y - v1.y * v2.x );
    }
 };
//---------------------------------------------------------------------------
inline Vector Normalize ( const Vector& v ) {
  return v / !v;
  }
//---------------------------------------------------------------------------
inline Vector& Clip ( Vector& v, double m = 0.0, double M = 1.0 ) {
  v.x = v.x < m ? m : v.x > M ? M : v.x;
  v.y = v.y < m ? m : v.y > M ? M : v.y;
  v.z = v.z < m ? m : v.z > M ? M : v.z;
  return v;
  }
//---------------------------------------------------------------------------
#endif
