//---------------------------------------------------------------------------
#include <math.h>
#include <vcl.h>
#include "Vector.h"
#include "Matrix.h"
#pragma hdrstop

#include "Glmain.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TFormMain *FormMain;

//---------------------------------------------------------------------------
__fastcall TFormMain::TFormMain(TComponent* Owner)
    : TForm(Owner)
{
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::FormCreate(TObject *Sender)
{
    hdc = GetDC(Handle);
    SetPixelFormatDescriptor();
    hrc = wglCreateContext(hdc);
    if(hrc == NULL)
    	ShowMessage(":-)~ hrc == NULL");
    if(wglMakeCurrent(hdc, hrc) == false)
    	ShowMessage("Could not MakeCurrent");
    w = ClientWidth;
    h = ClientHeight;

    glEnable(GL_DEPTH_TEST);
    glEnable(GL_NORMALIZE);

    SetupTextures();
    SetupLighting();

   glClearColor(1.0f, 1.0f, 1.0f, 1.0f);




}
//---------------------------------------------------------------------------
void __fastcall TFormMain::SetPixelFormatDescriptor()
{
    PIXELFORMATDESCRIPTOR pfd = {     //      
    	sizeof(PIXELFORMATDESCRIPTOR),   //  
        1,         // 
        PFD_DRAW_TO_WINDOW |  //  
        PFD_SUPPORT_OPENGL |  //  OpenGl
        PFD_DOUBLEBUFFER,     //   
        PFD_TYPE_RGBA,        // RGBA 
        24,                   // 24   
        0,0,0,0,0,0,          //  
        0,                    //  
        0,                    //  
        0,                    //  
        0,0,0,0,              //  
        32,                   //32-  
        0,                    //  
        0,                    //  
        PFD_MAIN_PLANE,       //  
        0,                    //
        0,0,                  //  
    };
    PixelFormat = ChoosePixelFormat(hdc, &pfd);
    //      
    SetPixelFormat(hdc, PixelFormat, &pfd); //  

}
//---------------------------------------------------------------------------
void __fastcall TFormMain::FormResize(TObject *Sender)
{

    GLfloat nRange = 4.0;

    w = ClientWidth;
    h = ClientHeight;

   if(h == 0)  //    ,    
    	h = 1;
    glViewport(0, 0, w, h);  //      
    glMatrixMode(GL_PROJECTION);  //   
    glLoadIdentity(); //  
    if (w <= h)
        glOrtho (-nRange, nRange, -nRange*h/w, nRange*h/w, -nRange, nRange);
    else
	glOrtho (-nRange*w/h, nRange*w/h, -nRange, nRange, -nRange, nRange);
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();  //  
            }
//---------------------------------------------------------------------------
void __fastcall TFormMain::RenderGLScene()
{
   glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    //     
    DrawObjects(); //    
    glFlush();
    //        

}
//---------------------------------------------------------------------------
double Z(double x,double y)
{
  double z=(sin(x*x)+cos(y*y))*x*y;
  return z;
}
//---------------------------------------------------------------------------
double dZx(double x,double y)
{
  double pr=2*x*x*y*cos(x*x)+(sin(x*x)+cos(y*y))*y;
  return pr;
}
//---------------------------------------------------------------------------
double dZy(double x,double y)
{
  double pr=(-2)*x*y*y*sin(y*y)+x*(sin(x*x)+cos(y*y));
  return pr;
}
//---------------------------------------------------------------------------
void Draw()
{
Vector Norm;
//glPolygonMode(GL_FRONT_AND_BACK,GL_LINE);
  for(double x=-0.8; x<0.8; x=x+0.03)
  {
    for(double y=-1.5; y<1.5; y=y+0.03)
    {
      glBegin (GL_QUADS);

      glTexCoord2d(0, 0);
      Norm=Vector(dZx(x,y),dZy(x,y),-1);
      glNormal3d(Norm.x,Norm.y,Norm.z);
      glVertex3d(x,y,Z(x,y));

      glTexCoord2d(0, 1);
      Norm=Vector(dZx(x+1,y),dZy(x+1,y),-1);
      glNormal3d(Norm.x,Norm.y,Norm.z);
      glVertex3d(x+1,y,Z(x+1,y));

      glTexCoord2d(1,1);
      Norm=Vector(dZx(x+1,y+1),dZy(x+1,y+1),-1);
      glNormal3d(Norm.x,Norm.y,Norm.z);
      glVertex3d(x+1,y+1,Z(x+1,y+1));

      glTexCoord2d(1,0);
      Norm=Vector(dZx(x,y+1),dZy(x,y+1),-1);
      glNormal3d(Norm.x,Norm.y,Norm.z);
      glVertex3d(x,y+1,Z(x,y+1));
      glEnd();
    }
  }
}
//---------------------------------------------------------------------------
// 
void __fastcall TFormMain::DrawObjects()
{
  glBindTexture(GL_TEXTURE_2D,texture1);
  Draw();

}

//---------------------------------------------------------------------------
void __fastcall TFormMain::FormPaint(TObject *Sender)
{
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    glFlush();
    DrawObjects();
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::FormDestroy(TObject *Sender)
{
    wglMakeCurrent(NULL, NULL);
    wglDeleteContext(hrc);
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::FormKeyDown(TObject *Sender, WORD &Key,
      TShiftState Shift)
{
    if(Key == VK_UP)
       { glRotatef(-5, 1.0, 0.0, 0.0);  }
    if(Key == VK_DOWN)
       { glRotatef(5, 1.0, 0.0, 0.0);}
    if(Key == VK_LEFT)
       { glRotatef(-5, 0.0, 1.0, 0.0); }
    if(Key == VK_RIGHT)
       { glRotatef(5, 0.0, 1.0, 0.0); }

    if(Key == VK_ADD)
       { glScalef(1, 1, 1.5); }

    if(Key == VK_SUBTRACT)
       { glScalef(1, 1, 0.5); }


}
//---------------------------------------------------------------------------
void __fastcall TFormMain::SetupTextures()
{
 bitmap = new Graphics::TBitmap;
    bitmap->LoadFromFile("pic.bmp");
    GLubyte bits[8][8][4];
    for(int i = 0; i < 8; i++)
    {
    	for(int j = 0; j < 8; j++)
        {
            bits[i][j][0]= (GLbyte)GetRValue(bitmap->Canvas->Pixels[i][j]);
            bits[i][j][1]= (GLbyte)GetGValue(bitmap->Canvas->Pixels[i][j]);
            bits[i][j][2]= (GLbyte)GetBValue(bitmap->Canvas->Pixels[i][j]);
            bits[i][j][3]= (GLbyte)255;
        }
    }
    glPixelStorei(GL_UNPACK_ALIGNMENT, 4);
    glGenTextures(1, &texture1);
    glBindTexture(GL_TEXTURE_2D, texture1);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
    glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, 8, 8, 0, GL_RGBA, GL_UNSIGNED_BYTE, bits);
    glEnable(GL_TEXTURE_2D);
    glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
}
//---------------------------------------------------------------------------

void __fastcall TFormMain::SetupLighting()
{
    GLfloat MaterialAmbient[] = {0.5, 0.5, 0.5, 1.0};
    GLfloat MaterialDiffuse[] = {1.0, 1.0, 1.0, 1.0};
    GLfloat MaterialSpecular[] = {1.0, 1.0, 1.0, 1.0};
    GLfloat MaterialShininess[] = {50.0};
    GLfloat AmbientLightPosition[] = {0.5, 1.0, -1.0, 0.0};
    GLfloat LightAmbient[] = {0.5, 0.5, 0.5, 1.0};

    glMaterialfv(GL_FRONT, GL_AMBIENT, MaterialAmbient);
    glMaterialfv(GL_FRONT, GL_DIFFUSE, MaterialDiffuse);
    glMaterialfv(GL_FRONT, GL_SPECULAR, MaterialSpecular);
    glMaterialfv(GL_FRONT, GL_SHININESS, MaterialShininess);
    glLightfv(GL_LIGHT0, GL_POSITION, AmbientLightPosition);
    glLightModelfv(GL_LIGHT_MODEL_AMBIENT, LightAmbient);

    glEnable(GL_LIGHTING);
    glEnable(GL_LIGHT0);
    glEnable(GL_COLOR_MATERIAL);
    glColorMaterial(GL_FRONT, GL_AMBIENT_AND_DIFFUSE);
    glShadeModel(GL_SMOOTH);
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::Timer1Timer(TObject *Sender)
{
   RenderGLScene();   // 
    SwapBuffers(hdc);  //  

}
//---------------------------------------------------------------------------

